@@ -1,22 +1,34 @@
-from grid.cli.grid_view import view
-from grid.cli.grid_logs import logs
-from grid.cli.grid_login import login
-from grid.cli.grid_train import train
-from grid.cli.grid_status import status
-from grid.cli.grid_cancel import cancel
-from grid.cli.grid_history import history
 from grid.cli.grid_artifacts import artifacts
+from grid.cli.grid_cancel import cancel
 from grid.cli.grid_credentials import credentials
-from grid.cli.grid_interactive import interactive
-from grid.cli.grid_deletecluster import deletecluster
-from grid.cli.grid_delete import delete
 from grid.cli.grid_datastores import datastores
-
+from grid.cli.grid_delete import delete
+from grid.cli.grid_deletecluster import deletecluster
+from grid.cli.grid_history import history
+from grid.cli.grid_interactive import interactive
+from grid.cli.grid_login import login
+from grid.cli.grid_logs import logs
 # SLURM
 from grid.cli.grid_slurm import slurm
+from grid.cli.grid_ssh_keys import ssh_keys
+from grid.cli.grid_status import status
+from grid.cli.grid_train import train
+from grid.cli.grid_view import view
 
 __all__ = [
-    'view', 'status', 'login', 'train', 'cancel', 'credentials', 'history',
-    'logs', 'interactive', 'artifacts', 'deletecluster', 'delete',
-    'datastores', 'slurm'
+    'view',
+    'status',
+    'login',
+    'train',
+    'cancel',
+    'credentials',
+    'history',
+    'logs',
+    'interactive',
+    'artifacts',
+    'deletecluster',
+    'delete',
+    'datastores',
+    'slurm',
+    'ssh_keys',
 ]
