@@ -1,17 +1,20 @@
-import yaml
-import click
 import random
-import grid.globals as env
 import sys
+from typing import Any, Dict, Optional
 
-from typing import Optional, Any, Dict
+import click
 from coolname import generate_slug
 
-from grid import Grid, Segment
+from grid import Grid
+from grid.cli.grid_credentials import print_creds_table
+from grid.cli.utilities import read_config
+from grid.cli.utilities import validate_config
+from grid.cli.utilities import validate_disk_size_callback
+import grid.globals as env
+from grid.tracking import Segment
+from grid.tracking import TrackingEvents
 from grid.types import WorkflowType
 from grid.utilities import is_experiment
-from grid.cli.grid_credentials import print_creds_table
-from grid.cli.grid_config import validate_config, validate_disk_size_callback
 
 
 def get_credentials(client: Grid,
@@ -55,7 +58,7 @@ def get_credentials(client: Grid,
                 f"to run on {credential['provider'].upper()}.")
             return credential
 
-    # If no credentials are available, raise an exception.
+    # If no default credentials are available, raise an exception.
     m = """
     Detected multiple credentials. Which would you like to use?
     """
@@ -118,40 +121,11 @@ def _check_ignore_warnings_flag(ctx, _param, value):
     return value
 
 
-def _overwrite_config(ctx, param, value):
-    """
-    Click callback that replaces values from a passed config
-    YML with the values passed from the CLI context.
-    """
-    patched_grid_config = None
-    grid_config = value
-    if grid_config:
-
-        #  Loads the YML file as passed by the
-        #  user.
-        try:
-            grid_config = yaml.safe_load(value.read())
-        except (yaml.scanner.ScannerError, yaml.reader.ReaderError) as e:
-            raise click.BadParameter('Could not load your YAML config file.',
-                                     e)
-
-        #  Adds required structure to the base
-        #  YML file, if that structure isn't there.
-        if 'compute' not in grid_config:
-            grid_config['compute'] = {}
-        if 'train' not in grid_config['compute']:
-            grid_config['compute']['train'] = {}
-
-        #  Generate a YML in-memory representation of
-        #  the edited config file.
-        patched_grid_config = yaml.dump(grid_config)
-
-    return grid_config or patched_grid_config
-
-
 def _print_submission_message(instance_type: str, provider: str,
                               credential_id: str, entrypoint: str,
-                              grid_name: str):
+                              grid_name: str, datastore_name: str,
+                              datastore_version: int,
+                              datastore_mount_dir: str):
     """Prints train submission message to the client."""
     message = f"""
         Run submitted!
@@ -166,24 +140,26 @@ def _print_submission_message(instance_type: str, provider: str,
         cloud_provider:          {provider}
         cloud_credentials:       {credential_id}
         grid_name:               {grid_name}
+        datastore_name:          {datastore_name}
+        datastore_version:       {datastore_version}
+        datastore_mount_dir:     {datastore_mount_dir}
         """
     click.echo(message)
 
 
 def _generate_default_grid_config(client: Grid,
                                   grid_credential: str,
-                                  entrypoint: str,
                                   instance_type: str,
                                   grid_strategy: Optional[str] = None,
                                   grid_trials: Optional[int] = None,
                                   gpus: Optional[int] = None,
                                   max_nodes: Optional[int] = None,
-                                  autoscale_interval: Optional[int] = None,
                                   disk_size: Optional[int] = None,
                                   memory: str = None,
                                   cpus: Optional[int] = None,
+                                  processes: Optional[int] = None,
                                   datastore_name: Optional[str] = None,
-                                  datastore_version: Optional[str] = None,
+                                  datastore_version: Optional[int] = None,
                                   datastore_mount_dir: Optional[str] = None):
     """
     Generates a new default config file for user if user hasn't
@@ -192,12 +168,6 @@ def _generate_default_grid_config(client: Grid,
     _default_credential = get_credentials(client=client,
                                           grid_credential=grid_credential)
 
-    if (not datastore_name) != (not datastore_version):
-        raise click.ClickException("""
-        To use datastore for this job, both datastore_name and datastore_version
-        must be set.
-        """)
-
     #  Relevant defaults for user.
     #  TODO: Pull from user profile when we have
     #  that information collected.
@@ -222,7 +192,8 @@ def _generate_default_grid_config(client: Grid,
                 30 * 60,  # 30 minutes before scaling down.
                 'disk_size': disk_size,
                 'memory': memory,
-                'cpus': cpus
+                'cpus': cpus,
+                'processes': processes
             }
         }
     }
@@ -272,7 +243,7 @@ def _check_is_valid_extension(ctx, _param, value):
               'config',
               type=click.File('r'),
               required=False,
-              callback=_overwrite_config,
+              callback=read_config,
               help='Path to Grid config YML')
 @click.option('--grid_name',
               '--g_name',
@@ -307,13 +278,12 @@ def _check_is_valid_extension(ctx, _param, value):
               type=int,
               required=False,
               help='Number of trials to run hyper parameter search')
-@click.option(
-    '--grid_instance_type',
-    '--g_instance_type',
-    'instance_type',
-    type=str,
-    default='g4dn.xlarge',  # 1 x T4
-    help='Instance type to start training session in')
+@click.option('--grid_instance_type',
+              '--g_instance_type',
+              'instance_type',
+              type=str,
+              default='t2.xlarge',
+              help='Instance type to start training session in')
 @click.option('--grid_gpus',
               '--g_gpus',
               'gpus',
@@ -329,6 +299,16 @@ def _check_is_valid_extension(ctx, _param, value):
     required=False,
     default=0,  # Default will be set to 1 after alerting user
     help='Number of CPUs to allocate per experiment')
+# TODO: Re-enable this when we want to enable mult-nodex
+#@click.option(
+#    '--grid_processes',
+#    '--g_processes',
+#    'processes',
+#    type=int,
+#    required=False,
+#    default=1,
+#    help='Number of training processes to run per experiment. Using larger than 1 ' \
+#         'will start a multi-node training')
 @click.option(
     '--grid_disk_size',
     '--g_disk_size',
@@ -379,7 +359,7 @@ def _check_is_valid_extension(ctx, _param, value):
                 callback=_check_is_valid_extension)
 @click.argument('script_args', nargs=-1, type=click.UNPROCESSED)
 @click.pass_context
-def train(ctx, script: str, entrypoint: str, config: Optional[click.File],
+def train(ctx, script: str, entrypoint: str, config: Optional[Dict],
           name: Optional[str], credential: Optional[str],
           strategy: Optional[str], trials: Optional[int], instance_type: str,
           gpus: Optional[int], disk_size: Optional[int],
@@ -402,11 +382,9 @@ def train(ctx, script: str, entrypoint: str, config: Optional[click.File],
 
     client = Grid()
 
-    # Connect list of user input back into string and send to Segment
-    if env.SEGMENT_TRACKING:
-        Segment().send(event="Run Created",
-                       properties={'user_input': " ".join(sys.argv)},
-                       user_id=client.user_id)
+    tracker = Segment()
+    tracker.send_event(event=TrackingEvents.RUN_CREATED,
+                       properties={'user_input': " ".join(sys.argv)})
 
     # make a fun random name when user does not pass in a name
     if name is None:
@@ -421,10 +399,10 @@ def train(ctx, script: str, entrypoint: str, config: Optional[click.File],
         _grid_config = _generate_default_grid_config(
             client=client,
             grid_credential=credential,
-            entrypoint=entrypoint,
             grid_strategy=strategy,
             grid_trials=trials,
             instance_type=instance_type,
+            processes=1,
             gpus=gpus,
             max_nodes=max_nodes,
             disk_size=disk_size,
@@ -443,24 +421,24 @@ def train(ctx, script: str, entrypoint: str, config: Optional[click.File],
     cpus = _grid_config['compute']['train'].get('cpus')
     gpus = _grid_config['compute']['train'].get('gpus')
     if not cpus and not gpus:
-        notice_str = click.style('NOTICE', fg='yellow')
-        cpu_warning = f"""
-        {notice_str}
+        notice_str = click.style('WARNING', fg='yellow')
+        cpu_warning = f"{notice_str} Neither a CPU or GPU number was specified. 1 CPU will be " + \
+                      "used as a default. To use N GPUs pass in '--grid_gpus N' flag."
 
-        Neither a CPU or GPU number was specified, so 1 CPU will be
-        used as a default.
-
-        Would you like to still continue running this job?
-
-     """
         _grid_config['compute']['train']['cpus'] = 1
+        # We will be showing this warning to users, but not
+        # issuing a prompt.
         if not env.IGNORE_WARNINGS:
-            click.confirm(cpu_warning, abort=True)
+            # click.confirm(cpu_warning, abort=True)
+            click.echo(cpu_warning)
 
-    _grid_config_yaml = yaml.dump(_grid_config)
+    # TODO: Re-enable this when we want to enable mult-nodex
+    #if processes and not entrypoint.endswith(".py") and processes > 1:
+    #    raise click.ClickException(
+    #        "Multiple processes is only supported for " + "python programs")
 
     #  Send to client.
-    client.train(config=_grid_config_yaml,
+    client.train(config=_grid_config,
                  kind=WorkflowType.SCRIPT,
                  run_name=name,
                  run_description=description,
@@ -474,4 +452,9 @@ def train(ctx, script: str, entrypoint: str, config: Optional[click.File],
         provider=_grid_config['compute']['provider']['vendor'],
         credential_id=_grid_config['compute']['provider']['credentials'],
         entrypoint=entrypoint,
-        grid_name=name)
+        grid_name=name,
+        datastore_name=_grid_config['compute']['train']['datastore_name'],
+        datastore_version=_grid_config['compute']['train']
+        ['datastore_version'],
+        datastore_mount_dir=_grid_config['compute']['train']
+        ['datastore_mount_dir'])
