@@ -1,17 +1,20 @@
 """Set of utility methods."""
+from datetime import datetime
+from datetime import timedelta
+from datetime import timezone
 import os
+from os import listdir
 import re
-import pytz
+from shlex import split
+from types import ModuleType
+from typing import List
 from urllib import parse
 
-from os import listdir
-from typing import List
-from types import ModuleType
-from google.cloud import storage
-from datetime import datetime, timedelta, timezone
+import click
 from dateutil.parser import parse as date_string_parse
+from google.cloud import storage
+import pytz
 from tzlocal import get_localzone
-from shlex import split
 
 
 class GCPStorage:  # pragma: no cover
@@ -85,8 +88,10 @@ def get_abs_time_difference(date_1: datetime, date_2: datetime) -> timedelta:
 
     # check if need to localize:
     if not date_1.tzinfo:
+        # skipcq: PYL-E1120
         date_1 = pytz.UTC.localize(dt=date_1)
     if not date_2.tzinfo:
+        # skipcq: PYL-E1120
         date_2 = pytz.UTC.localize(dt=date_2)
     diff = abs(date_2 - date_1)
     return diff
@@ -100,6 +105,14 @@ def convert_utc_string_to_local_datetime(date: str) -> datetime:
     return date_string_parse(date).replace(tzinfo=get_localzone())
 
 
+def string_format_date(date: datetime):
+    """
+    Get's a human readable string formatted version
+    of a datetime object.
+    """
+    return date.strftime("%Y-%m-%d %H:%M:%S%z")
+
+
 def string_format_timedelta(delta: timedelta) -> str:
     """
     Gets the iso8601 string format of a datetime.
@@ -192,7 +205,7 @@ def is_experiment(identifier: str) -> bool:
     ----------
     identifier: str
         Run or Experiment identifier to check.
-    
+
     Returns
     -------
     output: bool
@@ -213,13 +226,13 @@ def get_param_values(command: str) -> List[str]:
     Converts string parameters into a list of strings.
     This is useful for rendering such parameters into
     a table.
-    
+
     Parameters
     ----------
     command: str
         String representing invocation command alongside
         parameters.
-    
+
     Returns
     -------
     hparam_vals: List[str]
@@ -260,15 +273,62 @@ def get_experiment_duration_string(created_at: str, started_running_at: str,
 def get_graphql_url(url: str) -> str:
     """
     Appends a graphql to the end of the url if not included
-    
+
     Parameters
     ----------
     url: str
         URL potentially including graphql
-    
+
     Returns
     -------
     A url appended with graphql if not already included
     """
     return url if parse.urlparse(url).path.endswith(
         '/graphql') else parse.urljoin(url, "/graphql")
+
+
+def check_is_python_script(ctx, _param, value):
+    """Click callback that checks if a file is a Python script."""
+    if value is not None:
+        if not value.endswith('.py'):
+            raise click.BadParameter('You must provide a Python script. '
+                                     'No script detected.')
+
+        ctx.params['entrypoint'] = value
+        return value
+
+
+def check_run_name_is_valid(_ctx, _param, value):
+    """Click callback that checks if a Run contains reserved names."""
+    if value is not None:
+        if is_experiment(value):
+            raise click.BadParameter('Runs cannot end with "exp[0...n]".')
+
+        fail = False
+
+        #  Check if the input is alphanumeric.
+        _run_name = value.replace('-', '')
+        if not _run_name.isalnum():
+            fail = True
+
+        #  Check if the allowed `-` character is not used
+        #  at the end of the string.
+        elif value.endswith('-') or value.startswith('-'):
+            fail = True
+
+        #  Check that the run name does not contain any
+        #  uppercase characters.
+        elif any(x.isupper() for x in value):
+            fail = True
+
+        if fail:
+            raise click.BadParameter(f"""
+
+            Invalid Run name: {value}. Run names must be lower case
+            alphanumeric characters or '-', start with an alphabetic
+            character, and end with an alphanumeric character (e.g. 'my-name',
+            or 'abc-123').
+
+                """)
+
+    return value
