@@ -1,21 +1,27 @@
-import shutil
-import unittest
-
 from pathlib import Path
-from grid.downloader import Downloader, DownloadableObject
+import shutil
+
+from grid.downloader import DownloadableObject
+from grid.downloader import Downloader
 
 
-class ArtifactCallbacksTestCase(unittest.TestCase):
+class TestArtifactCallbacks:
     """Tests callbacks in grid artifacts."""
     @classmethod
-    def setUpClass(cls):
-        cls.test_dir = 'tests/data/download_path/nested_dir'
-
-    def tearDown(self):
-        # Deletes test directory with all files.
-        path = Path(self.test_dir)
+    def clean_test_directory(cls):
+        path = Path(cls.test_dir)
         shutil.rmtree(path, ignore_errors=True)
 
+    @classmethod
+    def setup_class(cls):
+        cls.test_dir = 'tests/data/download_path/nested_dir'
+        cls.clean_test_directory()
+
+    @classmethod
+    def teardown_class(cls):
+        # Deletes test directory with all files.
+        cls.clean_test_directory()
+
     def test_downloader_creates_dir(self):
         """Downloader.create_dir_tree creates dir structure in path."""
         Downloader.create_dir_tree(self.test_dir)
@@ -36,8 +42,8 @@ class ArtifactCallbacksTestCase(unittest.TestCase):
 
         # Download file
         D = Downloader(downloadable_objects=objects, base_dir=self.test_dir)
-        D.download()
+        assert D.download()
 
-        # Test that file as been downloaded
+        # Test that file as been downloaded.
         assert Path(self.test_dir).joinpath(directory_path).joinpath(
             filename).exists()
