@@ -1,21 +1,28 @@
-import os
 import csv
 import glob
 import json
+import os
+from pathlib import Path
+from unittest.mock import MagicMock
+from urllib.parse import urljoin
+
 import click
+from click.exceptions import ClickException
+from gql import Client
+import httpretty
 import pytest
 import requests
-import unittest
+from tests.mock_backend import GridAIBackenedTestServer
+from tests.mock_backend import resolvers
 import yaspin
 
-from pathlib import Path
-from tests.mock_backend import GridAIBackenedTestServer, resolvers
-from tests.utilities import monkeypatched
-
 import grid.client as grid
+from grid.exceptions import AuthenticationError
+from grid.exceptions import TrainError
 import grid.globals as env
-from grid.types import WorkflowType, ObservableType
-from grid.exceptions import TrainError, AuthenticationError
+from grid.metadata import __version__
+from grid.types import ObservableType
+from grid.types import WorkflowType
 
 #  Prevents Click from opening the browser.
 click.launch = lambda x: True
@@ -26,6 +33,9 @@ def monkey_patch_observables():
     class MonkeyPatchedObservable:
         key = 'getRuns'
 
+        def __init__(self, *args, **kwargs):
+            pass
+
         def get(self, *args, **kwarrgs):
             return {
                 self.key: [{
@@ -54,10 +64,10 @@ class MonkeyPatchClient:
         raise Exception("{'message': 'test exception'}")
 
 
-class GridClientTestCase(unittest.TestCase):
+class TestGridClient:
     """Unit tests for the Grid class."""
     @classmethod
-    def setUpClass(cls):
+    def setup_class(cls):
         #  Monkey patches the GraphQL client to read from a local schema.
         def monkey_patch_client(self):
             self.client = GridAIBackenedTestServer()
@@ -107,7 +117,7 @@ class GridClientTestCase(unittest.TestCase):
         self.remove_status_files()
         self.remove_credentials_file()
 
-    def tearDown(self):
+    def teardown(self):
         self.remove_env()
 
         #  Removes test credentials added to home path.
@@ -134,7 +144,7 @@ class GridClientTestCase(unittest.TestCase):
             assert key in G.headers.keys()
 
         os.environ['GRID_CREDENTIAL_PATH'] = 'fake-path'
-        with self.assertRaises(click.ClickException):
+        with pytest.raises(click.ClickException):
             grid.Grid()
 
         self.remove_env()
@@ -142,14 +152,18 @@ class GridClientTestCase(unittest.TestCase):
     def test_client_raises_exception_if_creds_path_not_found(self):
         """Client raises exception if credentials path not found"""
         credentials_path = 'tests/data/foo.json'
-        with self.assertRaises(click.ClickException):
+        with pytest.raises(click.ClickException):
             grid.Grid(credential_path=credentials_path)
 
     def test_nested_path_is_parsed_correctly(self):
         """Tests that we can add the Git root path to a script"""
         result = grid.Grid._add_git_root_path(entrypoint='foo.py')
         path_elems = result.split(os.path.sep)
-        self.assertListEqual(path_elems[-2:], ['grid-cli', 'foo.py'])
+
+        actual = path_elems[-2:]
+        expected = ['grid-cli', 'foo.py']
+        assert len(actual) == len(expected)
+        assert all(a == b for a, b in zip(actual, expected))
 
     def test_client_loads_credentials_from_default_path(self):
         """Client loads credentials from default path"""
@@ -171,12 +185,12 @@ class GridClientTestCase(unittest.TestCase):
 
     def test_client_raises_error_if_no_creds_available(self):
         """Client loads credentials from default path"""
-        with self.assertRaises(click.ClickException):
+        with pytest.raises(click.ClickException):
             grid.Grid(credential_path=self.creds_path_incorrect)
 
         # TODO: test not working
-        # with self.assertRaises(click.ClickException):
-        #     # with monkeypatched(grid.Grid, 'grid_credentials_path', self.creds_path_incorrect):
+        # with pytest.raises(click.ClickException):
+        #     # monkeypatch.setattr(grid.Grid, 'grid_credentials_path', self.creds_path_incorrect):
         #     grid.Grid.grid_credentials_path = self.creds_path_incorrect
         #     G = grid.Grid(load_local_credentials=False)
         #     G._set_local_credentials()
@@ -192,7 +206,7 @@ class GridClientTestCase(unittest.TestCase):
         for key in self.grid_header_keys:
             assert key not in G.headers.keys()
 
-    def test_train(self):
+    def test_train(self, monkeypatch):
         """grid.Grid().train() executes a training operation correctly."""
         G = grid.Grid(credential_path=self.creds_path,
                       load_local_credentials=False)
@@ -200,9 +214,9 @@ class GridClientTestCase(unittest.TestCase):
         G.train(**self.train_kwargs)
 
         # We want to ignore global settings to test debugging.
-        with monkeypatched(grid.Grid, '_load_global_settings', lambda: True):
-            env.DEBUG = True
-            G.train(**self.train_kwargs)
+        monkeypatch.setattr(grid.Grid, '_load_global_settings', lambda: True)
+        env.DEBUG = True
+        G.train(**self.train_kwargs)
 
     def test_train_raises_exception_blueprint(self):
         """
@@ -212,7 +226,7 @@ class GridClientTestCase(unittest.TestCase):
         G = grid.Grid(credential_path=self.creds_path,
                       load_local_credentials=False)
         env.IGNORE_WARNINGS = True
-        with self.assertRaises(TrainError):
+        with pytest.raises(TrainError):
             G.train(**{**self.train_kwargs, 'kind': WorkflowType.BLUEPRINT})
 
     def test_train_raises_exception_if_query_fails(self):
@@ -222,7 +236,7 @@ class GridClientTestCase(unittest.TestCase):
         G._check_user_github_token = lambda: True
         env.IGNORE_WARNINGS = True
 
-        with self.assertRaises(click.ClickException):
+        with pytest.raises(click.ClickException):
             G.train(**self.train_kwargs)
 
     def test_status_returns_results(self):
@@ -264,7 +278,7 @@ class GridClientTestCase(unittest.TestCase):
                                         download_dir='tests/data')
 
     @staticmethod
-    def test_download_experiment_artifacts_handles_exception():
+    def test_download_experiment_artifacts_handles_exception(monkeypatch):
         """
         grid.Grid().download_experiment_artifacts() handles exception for when
         the GraphQL query returns an error.
@@ -274,11 +288,11 @@ class GridClientTestCase(unittest.TestCase):
         G._init_client()
 
         monkey_patch_get_artifacts = lambda: {'errors': 'error'}
-        with monkeypatched(resolvers, 'get_artifacts',
-                           monkey_patch_get_artifacts):
-            with pytest.raises(click.ClickException):
-                G.download_experiment_artifacts(experiment_id=experiment_id,
-                                                download_dir='tests/data')
+        monkeypatch.setattr(resolvers, 'get_artifacts',
+                            monkey_patch_get_artifacts)
+        with pytest.raises(click.ClickException):
+            G.download_experiment_artifacts(experiment_id=experiment_id,
+                                            download_dir='tests/data')
 
     def test_user_id(self):
         """Returns same User ID as specified in credentials file."""
@@ -319,7 +333,7 @@ class GridClientTestCase(unittest.TestCase):
         assert result == True
 
     @staticmethod
-    def test_check_user_github_token_raises_exception():
+    def test_check_user_github_token_raises_exception(monkeypatch):
         """
         grid.Grid()._check_user_github_token() raises exception if no
         Github token is available.
@@ -332,16 +346,16 @@ class GridClientTestCase(unittest.TestCase):
             return {'hasValidToken': False}
 
         with pytest.raises(click.exceptions.ClickException):
-            with monkeypatched(resolvers, 'check_user_github_token',
-                               monkey_patch_token):
-                G._check_user_github_token()
+            monkeypatch.setattr(resolvers, 'check_user_github_token',
+                                monkey_patch_token)
+            G._check_user_github_token()
 
         with pytest.raises(AuthenticationError):
-            with monkeypatched(GridAIBackenedTestServer, 'execute',
-                               monkey_patch_execute):
-                # Prevents browser from opening
-                with monkeypatched(click, 'launch', lambda x: True):
-                    G._check_user_github_token()
+            monkeypatch.setattr(GridAIBackenedTestServer, 'execute',
+                                monkey_patch_execute)
+            # Prevents browser from opening
+            monkeypatch.setattr(click, 'launch', lambda x: True)
+            G._check_user_github_token()
 
     @staticmethod
     def test_add_git_root_path():
@@ -359,17 +373,17 @@ class GridClientTestCase(unittest.TestCase):
         G.login(username='test-user', key='test-key')
 
     @staticmethod
-    def test_login_handles_error():
+    def test_login_handles_error(monkeypatch):
         """Grid.login() handles HTTPError exception."""
         G = grid.Grid(load_local_credentials=False)
 
-        with pytest.raises(AuthenticationError):
-            with monkeypatched(GridAIBackenedTestServer, 'execute',
-                               monkey_patch_execute):
-                G.login(username='test-user', key='test-key')
+        with pytest.raises(ClickException):
+            monkeypatch.setattr(GridAIBackenedTestServer, 'execute',
+                                monkey_patch_execute)
+            G.login(username='test-user', key='test-key')
 
     @staticmethod
-    def test_delete_cluster():
+    def test_delete_cluster(monkeypatch):
         """Grid().delete_cluster() deletes a cluster."""
         G = grid.Grid(load_local_credentials=False)
         G._init_client()
@@ -379,21 +393,21 @@ class GridClientTestCase(unittest.TestCase):
         def monkey_patch_delete_cluster(*args, **kwargs):
             return {'success': False, 'message': ''}
 
-        with monkeypatched(resolvers, 'delete_cluster',
-                           monkey_patch_delete_cluster):
-            assert not G.delete_cluster(cluster_id='test-cluster')
+        monkeypatch.setattr(resolvers, 'delete_cluster',
+                            monkey_patch_delete_cluster)
+        assert not G.delete_cluster(cluster_id='test-cluster')
 
     @staticmethod
-    def test_delete_cluster_handles_api_error():
+    def test_delete_cluster_handles_api_error(monkeypatch):
         """Grid().delete_cluster() handles API error."""
         monkey_patch_delete_cluster = lambda: {}
 
         G = grid.Grid(load_local_credentials=False)
         G._init_client()
-        with monkeypatched(resolvers, 'delete_cluster',
-                           monkey_patch_delete_cluster):
-            with pytest.raises(click.ClickException):
-                G.delete_cluster(cluster_id='test-cluster')
+        monkeypatch.setattr(resolvers, 'delete_cluster',
+                            monkey_patch_delete_cluster)
+        with pytest.raises(click.ClickException):
+            G.delete_cluster(cluster_id='test-cluster')
 
     # @staticmethod
     # def test_upload_datastore():
@@ -404,7 +418,7 @@ class GridClientTestCase(unittest.TestCase):
     #     def monkey_patch_upload(*args, **kwargs):
     #         return True
 
-    #     with monkeypatched(grid.S3DatastoreUploader, 'upload',
+    #     monkeypatch.setattr(grid.S3DatastoreUploader, 'upload',
     #                        monkey_patch_upload):
     #         result = G.upload_datastore(
     #             source_dir='./tests/data/datastore_test/',
@@ -429,7 +443,6 @@ class GridClientTestCase(unittest.TestCase):
         with pytest.raises(click.ClickException):
             G.upload_datastore(source_dir='./tests/data/datastore_test/',
                                name='datastore-test',
-                               version='v0',
                                credential_id='test-credential')
 
     @staticmethod
@@ -441,7 +454,7 @@ class GridClientTestCase(unittest.TestCase):
         assert result['getExperimentDetails']['status'] == 'succeeded'
 
     @staticmethod
-    def test_create_interactive_node():
+    def test_create_interactive_node(monkeypatch):
         G = grid.Grid(load_local_credentials=False)
         G._init_client()
 
@@ -452,12 +465,11 @@ class GridClientTestCase(unittest.TestCase):
         def monkey_path_create(*args, **kwargs):
             return {'success': False, 'message': ''}
 
-        with monkeypatched(resolvers, 'create_interactive_node',
-                           monkey_path_create):
-            env.DEBUG = True
-            with pytest.raises(click.ClickException):
-                G.create_interactive_node(name='test-interactive-id',
-                                          config='')
+        monkeypatch.setattr(resolvers, 'create_interactive_node',
+                            monkey_path_create)
+        env.DEBUG = True
+        with pytest.raises(click.ClickException):
+            G.create_interactive_node(name='test-interactive-id', config='')
 
     @staticmethod
     def test_create_interactive_node_handles_error():
@@ -469,23 +481,23 @@ class GridClientTestCase(unittest.TestCase):
             G.create_interactive_node(name='test-interactive-id', config='')
 
     @staticmethod
-    def test_delete_interactive_node():
+    def test_delete_interactive_node(monkeypatch):
         G = grid.Grid(load_local_credentials=False)
         G._init_client()
 
         result = G.delete_interactive_node(
-            interactive_node_id='test-interactive-id')
+            interactive_node_name='test-interactive-id')
         assert result
 
         def monkey_path_delete(*args, **kwargs):
             return {'success': False, 'message': ''}
 
-        with monkeypatched(resolvers, 'delete_interactive_node',
-                           monkey_path_delete):
-            env.DEBUG = True
-            with pytest.raises(click.ClickException):
-                G.delete_interactive_node(
-                    interactive_node_id='test-interactive-id')
+        monkeypatch.setattr(resolvers, 'delete_interactive_node',
+                            monkey_path_delete)
+        env.DEBUG = True
+        with pytest.raises(click.ClickException):
+            G.delete_interactive_node(
+                interactive_node_name='test-interactive-id')
 
     @staticmethod
     def test_delete_interactive_node_handles_error():
@@ -495,7 +507,7 @@ class GridClientTestCase(unittest.TestCase):
 
         with pytest.raises(click.ClickException):
             G.delete_interactive_node(
-                interactive_node_id='test-interactive-id')
+                interactive_node_name='test-interactive-id')
 
     @staticmethod
     def test_delete():
@@ -509,7 +521,7 @@ class GridClientTestCase(unittest.TestCase):
         assert result
 
     @staticmethod
-    def test_delete_handles_errors():
+    def test_delete_handles_errors(monkeypatch):
         G = grid.Grid(load_local_credentials=False)
         G._init_client()
         G._check_user_github_token = lambda: True
@@ -517,17 +529,17 @@ class GridClientTestCase(unittest.TestCase):
         def mp_delete_object(**kwargs):
             return {'success': False, 'message': None}
 
-        with monkeypatched(resolvers, 'delete_experiment', mp_delete_object):
-            with pytest.raises(click.ClickException):
-                G.delete(experiment_id='test-experiment-id')
+        monkeypatch.setattr(resolvers, 'delete_experiment', mp_delete_object)
+        with pytest.raises(click.ClickException):
+            G.delete(experiment_id='test-experiment-id')
 
         G.client = MonkeyPatchClient()
         with pytest.raises(click.ClickException):
             G.delete(experiment_id='test-experiment-id')
 
-        with monkeypatched(resolvers, 'delete_run', mp_delete_object):
-            with pytest.raises(click.ClickException):
-                G.delete(experiment_id='test-run-id')
+        monkeypatch.setattr(resolvers, 'delete_run', mp_delete_object)
+        with pytest.raises(click.ClickException):
+            G.delete(experiment_id='test-run-id')
 
         G.client = MonkeyPatchClient()
         with pytest.raises(click.ClickException):
@@ -538,14 +550,14 @@ class GridClientTestCase(unittest.TestCase):
         G = grid.Grid(load_local_credentials=False)
         G._init_client()
 
-        result = G.cancel(run_id='test-run-id')
+        result = G.cancel(run_name='test-run-id')
         assert result is True
 
         result = G.cancel(experiment_id='test-experiment-id')
         assert result is True
 
     @staticmethod
-    def test_cancel_handles_errors():
+    def test_cancel_handles_errors(monkeypatch):
         G = grid.Grid(load_local_credentials=False)
         G.client = MonkeyPatchClient()
         G._check_user_github_token = lambda: True
@@ -553,13 +565,13 @@ class GridClientTestCase(unittest.TestCase):
         def monkey_patch_experiment_details(**kwargs):
             return 'failed'
 
-        with monkeypatched(G, 'experiment_details',
-                           monkey_patch_experiment_details):
-            with pytest.raises(click.ClickException):
-                G.cancel(run_id='test-run-id')
+        monkeypatch.setattr(G, 'experiment_details',
+                            monkey_patch_experiment_details)
+        with pytest.raises(click.ClickException):
+            G.cancel(run_name='test-run-id')
 
     @staticmethod
-    def test_cancel_experiments():
+    def test_cancel_experiments(monkeypatch):
         G = grid.Grid(load_local_credentials=False)
         G._init_client()
 
@@ -568,8 +580,338 @@ class GridClientTestCase(unittest.TestCase):
         def mp_cancel_experiments(*args, **kwargs):
             return {'success': False, 'message': ''}
 
-        with monkeypatched(resolvers, 'cancel_experiment',
-                           mp_cancel_experiments):
-            experiments = [{'experimentId': 'test0', 'status': False}]
+        monkeypatch.setattr(resolvers, 'cancel_experiment',
+                            mp_cancel_experiments)
+        experiments = [{'experimentId': 'test0', 'status': False}]
+        with pytest.raises(click.ClickException):
+            G._cancel_experiments(experiments=experiments, spinner=spinner)
+
+    def test_history(self):
+        G = grid.Grid(credential_path=self.creds_path,
+                      load_local_credentials=False)
+        gql = MagicMock(Client, autospec=True)
+        G.client = gql
+
+        observable = MagicMock()
+        observable_fact = MagicMock(return_value=observable)
+        G.available_observables[ObservableType.EXPERIMENT] = observable_fact
+        observable.get_history = MagicMock(return_value={"a": "b"})
+        assert G.history(
+            kind=ObservableType.EXPERIMENT,
+            identifiers=["foo"],
+        ) == {
+            "a": "b"
+        }
+        observable_fact.assert_called_once()
+        observable_fact.assert_called_once_with(client=G.client,
+                                                identifier="foo")
+        observable.get_history.assert_called_once()
+
+        observable = MagicMock()
+        observable_fact = MagicMock(return_value=observable)
+        G.available_observables[ObservableType.RUN] = observable_fact
+        observable.get_history = MagicMock(return_value={"a": "b"})
+        assert G.history(kind=ObservableType.RUN, ) == {"a": "b"}
+        observable_fact.assert_called_once()
+        observable_fact.assert_called_once_with(client=G.client)
+        observable.get_history.assert_called_once()
+
+        observable = MagicMock()
+        observable_fact = MagicMock(return_value=observable)
+        G.available_observables[ObservableType.RUN] = observable_fact
+        observable.get_history = MagicMock(return_value={"a": "b"})
+        assert G.history() == {"a": "b"}
+        observable_fact.assert_called_once()
+        observable_fact.assert_called_once_with(client=G.client)
+        observable.get_history.assert_called_once()
+
+        with pytest.raises(click.BadArgumentUsage):
+            G.history(kind=ObservableType.INTERACTIVE)
+
+        with pytest.raises(click.BadArgumentUsage):
+            G.history(kind=ObservableType.CLUSTER)
+
+    def test_status(self):
+        G = grid.Grid(credential_path=self.creds_path,
+                      load_local_credentials=False)
+        gql = MagicMock(Client, autospec=True)
+        G.client = gql
+
+        observable = MagicMock()
+        observable_fact = MagicMock(return_value=observable)
+        G.available_observables[ObservableType.EXPERIMENT] = observable_fact
+        observable.get = MagicMock(return_value={"a": "b"})
+        assert G.status(
+            kind=ObservableType.EXPERIMENT,
+            identifiers=["foo"],
+        ) == {
+            "a": "b"
+        }
+        observable_fact.assert_called_once()
+        observable_fact.assert_called_once_with(client=G.client,
+                                                identifier="foo")
+        observable.get.assert_called_once()
+
+        observable = MagicMock()
+        observable_fact = MagicMock(return_value=observable)
+        G.available_observables[ObservableType.RUN] = observable_fact
+        observable.get = MagicMock(return_value={"a": "b"})
+        assert G.status(kind=ObservableType.RUN, ) == {"a": "b"}
+        observable_fact.assert_called_once()
+        observable_fact.assert_called_once_with(client=G.client)
+        observable.get.assert_called_once()
+
+        observable = MagicMock()
+        observable_fact = MagicMock(return_value=observable)
+        G.available_observables[ObservableType.RUN] = observable_fact
+        observable.get = MagicMock(return_value={"a": "b"})
+        assert G.status() == {"a": "b"}
+        observable_fact.assert_called_once()
+        observable_fact.assert_called_once_with(client=G.client)
+        observable.get.assert_called_once()
+
+        observable = MagicMock()
+        observable_fact = MagicMock(return_value=observable)
+        G.available_observables[ObservableType.INTERACTIVE] = observable_fact
+        observable.get = MagicMock(return_value={"a": "b"})
+        assert G.status(kind=ObservableType.INTERACTIVE) == {"a": "b"}
+        observable_fact.assert_called_once()
+        observable_fact.assert_called_once_with(client=G.client)
+        observable.get.assert_called_once()
+
+        with pytest.raises(click.BadArgumentUsage):
+            G.status(kind=ObservableType.CLUSTER)
+
+    def test_experiment_logs_queued(self):
+        G = grid.Grid(credential_path=self.creds_path,
+                      load_local_credentials=False)
+
+        G.client = MagicMock()
+        G.experiment_details = MagicMock(
+            return_value={'getExperimentDetails': {
+                'status': 'queued'
+            }})
+        G.experiment_logs('test')
+        G.client.execute.assert_not_called()
+
+    def test_experiment_logs_succeeded(self):
+        G = grid.Grid(credential_path=self.creds_path,
+                      load_local_credentials=False)
+
+        G.client = MagicMock(Client, autospec=True)
+        G.experiment_details = MagicMock(
+            return_value={'getExperimentDetails': {
+                'status': 'succeeded'
+            }})
+        G.client.execute = MagicMock(
+            return_value={
+                'getArchiveExperimentLogs': {
+                    'totalPages':
+                    2,
+                    'lines': [{
+                        "message": "foo",
+                        "timestamp": "date1"
+                    }, {
+                        "message": "bar",
+                        "timestamp": "date2"
+                    }]
+                }
+            })
+        G.experiment_logs('test')
+        G.client.execute.assert_called_once()
+
+    def test_experiment_logs_running(self):
+        G = grid.Grid(credential_path=self.creds_path,
+                      load_local_credentials=False)
+
+        G.client = MagicMock(Client, autospec=True)
+        G._init_client = MagicMock()
+        G.experiment_details = MagicMock(
+            return_value={'getExperimentDetails': {
+                'status': 'running'
+            }})
+        G.client.subscribe = MagicMock(return_value=[{
+            'getLiveExperimentLogs': [{
+                "message": "foo",
+                "timestamp": "date1"
+            }, {
+                "message": "bar",
+                "timestamp": "date2"
+            }]
+        }])
+        G.experiment_logs('test')
+        G._init_client.assert_called_once()
+        G.client.subscribe.assert_called_once()
+
+    @httpretty.activate
+    def test_check_version_compatibility(self, capsys, monkeypatch):
+        G = grid.Grid(credential_path=self.creds_path,
+                      load_local_credentials=False)
+
+        url = env.GRID_URL.replace('graphql', '')
+        httpretty.register_uri(httpretty.GET,
+                               urljoin(url, 'metadata'),
+                               body=json.dumps({'version': __version__}))
+        G._check_version_compatibility()
+
+        captured = capsys.readouterr()
+        assert 'pip install' not in captured.out
+
+        httpretty.register_uri(httpretty.GET,
+                               urljoin(url, 'metadata'),
+                               body=json.dumps({'version': 'v100.1.1'}))
+
+        try:
+            G._check_version_compatibility()
+        except Exception as e:
+            captured = capsys.readouterr()
+            assert 'pip install' in captured.out
+
             with pytest.raises(click.ClickException):
-                G._cancel_experiments(experiments=experiments, spinner=spinner)
+                raise e
+
+        # Check that version check skips
+        monkeypatch.setattr(env, 'SKIP_VERSION_CHECK', True)
+        G._check_version_compatibility()
+
+    @staticmethod
+    def test_gen_ssh_config():
+        assert grid.Grid._gen_ssh_config([],
+                                         "",
+                                         start_marker="#START",
+                                         end_marker="#END") == '\n'.join([
+                                             "#START",
+                                             "#END",
+                                         ])
+
+        assert grid.Grid._gen_ssh_config([],
+                                         "happy little config",
+                                         start_marker="#START",
+                                         end_marker="#END") == '\n'.join([
+                                             "happy little config",
+                                             "#START",
+                                             "#END",
+                                         ])
+
+        with pytest.raises(ValueError):
+            grid.Grid._gen_ssh_config([],
+                                      '\n'.join(["#START"]),
+                                      start_marker="#START",
+                                      end_marker="#END")
+
+        with pytest.raises(ValueError):
+            grid.Grid._gen_ssh_config([],
+                                      '\n'.join(["#END"]),
+                                      start_marker="#START",
+                                      end_marker="#END")
+
+        with pytest.raises(ValueError):
+            grid.Grid._gen_ssh_config([],
+                                      '\n'.join([
+                                          "#START",
+                                          "#END",
+                                          "#START",
+                                      ]),
+                                      start_marker="#START",
+                                      end_marker="#END")
+
+        with pytest.raises(ValueError):
+            grid.Grid._gen_ssh_config([],
+                                      '\n'.join([
+                                          "#START",
+                                          "#START",
+                                          "#END",
+                                      ]),
+                                      start_marker="#START",
+                                      end_marker="#END")
+
+        with pytest.raises(ValueError):
+            grid.Grid._gen_ssh_config([],
+                                      '\n'.join([
+                                          "#END",
+                                          "#START",
+                                      ]),
+                                      start_marker="#START",
+                                      end_marker="#END")
+
+        assert grid.Grid._gen_ssh_config(
+            [
+                {
+                    'name': 'ix1',
+                    'port': 233,
+                    'hostname': "ix1.grid.ai"
+                },
+                {
+                    'name': 'ix2',
+                    'port': 433,
+                    'hostname': "ix2.grid.ai"
+                },
+            ],
+            '\n'.join([
+                "happy little config",
+                "#START",
+                "#END",
+            ]),
+            start_marker="#START",
+            end_marker="#END") == '\n'.join([
+                "happy little config",
+                "#START",
+                "Host ix1",
+                "    User jovyan",
+                "    Hostname ix1.grid.ai",
+                "    Port 233",
+                "    StrictHostKeyChecking accept-new",
+                "Host ix2",
+                "    User jovyan",
+                "    Hostname ix2.grid.ai",
+                "    Port 433",
+                "    StrictHostKeyChecking accept-new",
+                "#END",
+            ])
+
+    @staticmethod
+    def test_gen_ssh_config_idempotent():
+        assert grid.Grid._gen_ssh_config(
+            [
+                {
+                    'name': 'ix1',
+                    'port': 233,
+                    'hostname': "ix1.grid.ai"
+                },
+                {
+                    'name': 'ix2',
+                    'port': 433,
+                    'hostname': "ix2.grid.ai"
+                },
+            ],
+            '\n'.join([
+                "happy little config",
+                "#START",
+                "Host ix1",
+                "    User jovyan",
+                "    Hostname ix1.grid.ai",
+                "    Port 233",
+                "    StrictHostKeyChecking accept-new",
+                "Host ix2",
+                "    User jovyan",
+                "    Hostname ix2.grid.ai",
+                "    Port 433",
+                "    StrictHostKeyChecking accept-new",
+                "#END",
+            ]),
+            start_marker="#START",
+            end_marker="#END") == '\n'.join([
+                "happy little config",
+                "#START",
+                "Host ix1",
+                "    User jovyan",
+                "    Hostname ix1.grid.ai",
+                "    Port 233",
+                "    StrictHostKeyChecking accept-new",
+                "Host ix2",
+                "    User jovyan",
+                "    Hostname ix2.grid.ai",
+                "    Port 433",
+                "    StrictHostKeyChecking accept-new",
+                "#END",
+            ])
